#include "stdafx.h"
#include "resource.h"
#include <windows.h>
#include <commctrl.h>
#include "NumberBox.h"
#include "PsxDlgProc.h"
#include "CommonDlg.h"
#include "PsExe.h"
#include "Savestate.h"
#include "PsxAddress.h"

const unsigned int ConvStrCount = 2;
extern char* ConvStrings[ConvStrCount*2] = 
							{ "ePSXe Savestate", 
							  "PSX RAM",
							  "PS-EXE Address",
							  "PSX RAM" };

const unsigned int DefaultPsExeLoadAddress = 0x80010000;
const unsigned int DefaultEpsxeRamAddress  = 0x1BA;
TCHAR buf[255];

bool canupdate;

HWND hPsExeLoad, hSsLoad;
HWND hConv1Text, hConv2Text;
HWND hConv1, hConv2;
unsigned int ConvOption;

unsigned int PsExeLoadAddress;
unsigned int EpsxeRamAddress;
unsigned int Conv1Value, Conv2Value;

BOOL CALLBACK PsxDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	static char PsExeFilename[MAX_PATH];
	static char SavestateFilename[MAX_PATH];
	HICON hIcon;

	switch(message)
	{
	case WM_INITDIALOG:
		InitCommonControlsEx(0);
		canupdate = false;
		hIcon = LoadIcon((HINSTANCE)GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_PSXADDRESS));
		SendMessage(hDlg, WM_SETICON, ICON_BIG, (LPARAM)hIcon);
		SendMessage(hDlg, WM_SETICON, ICON_SMALL, (LPARAM)hIcon);
		DestroyIcon(hIcon);
		PsExeLoadAddress = DefaultPsExeLoadAddress;
		EpsxeRamAddress = DefaultEpsxeRamAddress;
		Conv1Value = DefaultEpsxeRamAddress;
		Conv2Value = 0;

		// Init conversion type groupbox items
		hPsExeLoad = GetDlgItem(hDlg, IDC_PSEXELOAD);
		hSsLoad = GetDlgItem(hDlg, IDC_SSLOAD);
		HookEditToNumberEdit(hPsExeLoad);
		HookEditToNumberEdit(hSsLoad);
		wsprintf(buf, "%08X", DefaultPsExeLoadAddress);
		SetWindowText(hPsExeLoad, buf);
		wsprintf(buf, "%08X", DefaultEpsxeRamAddress);
		SetWindowText(hSsLoad, buf);
		CheckRadioButton(hDlg, IDC_SSRAM, IDC_EXERAM, IDC_SSRAM);
		ConvOption = 0;

		hConv1Text = GetDlgItem(hDlg, IDC_CONV1TEXT);
		hConv2Text = GetDlgItem(hDlg, IDC_CONV2TEXT);
		hConv1     = GetDlgItem(hDlg, IDC_CONV1);
		hConv2     = GetDlgItem(hDlg, IDC_CONV2);
		HookEditToNumberEdit(hConv1);
		HookEditToNumberEdit(hConv2);

		UpdateStaticText();
		wsprintf(buf, "%08X", DefaultEpsxeRamAddress);
		SetWindowText(hConv1, buf);
		SetWindowText(hConv2, "00000000");
		canupdate = true;
		return true;

	case WM_CLOSE:
		EndDialog(hDlg, NULL);
		return true;

	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
		case IDC_CONV1:
			if(HIWORD(wParam) == EN_CHANGE && canupdate)
			{
				Refresh(true);
				return true;
			}
			break;
		case IDC_CONV2:
			if(HIWORD(wParam) == EN_CHANGE && canupdate)
			{
				Refresh(false);
				return true;
			}
			break;
		case IDC_SSLOAD:
			if(HIWORD(wParam) == EN_CHANGE && canupdate)
			{
				Refresh(true);
				return true;
			}
			break;
		case IDC_PSEXELOAD:
			if(HIWORD(wParam) == EN_CHANGE && canupdate)
			{
				Refresh(true);
				return true;
			}
			break;

		case IDC_SSDETECT: // Get loading offset for savestate
			{
				EPSXE_SAVESTATE Savestate;
				memset(&Savestate, 0, sizeof(EPSXE_SAVESTATE));
				if(!OpenFileBox(hDlg, SavestateFilename, MAX_PATH, "ePSXe Savestate *.*\0*.*\0"))
					return true; // User cancelled

				DWORD res = OpenEpsxeSavestate(SavestateFilename, &Savestate);
				if(res == SS_NOFILE)
				{
					MessageBox(hDlg, "Savestate file does not exist", "File open error", MB_OK);
					return true;
				}
				else if(res == SS_BADFILE)
				{
					MessageBox(hDlg, "Savestate file is not a gzipped ePSXe savestate or is corrupt", "File open error", MB_OK);
					return true;
				}
				else if(res == SS_MEMERR)
				{
					MessageBox(hDlg, "File could not be read due to a memory error or savestate is not in a gzip archive", "Memory error", MB_OK);
					return true;
				}

				wsprintf(buf, "%08X", Savestate.PsxMem.data);
				SetWindowText(hSsLoad, buf);
				Refresh(true);
			}

			return true;

		case IDC_EXEDETECT: // Get loading offset for PS-EXE
			{
				PSEXE_HEADER ExeHeader;
				if(!OpenFileBox(hDlg, PsExeFilename, MAX_PATH, "PS-EXE *.*\0*.*\0"))
					return true; // User cancelled

				OpenPsExeFile(PsExeFilename, &ExeHeader);
				wsprintf(buf, "%08X", ExeHeader.t_addr);
				SetWindowText(hPsExeLoad, buf);
				Refresh(true);
			}
			return true;
		case IDC_SSRAM:
			ConvOption = 0;
			CheckRadioButton(hDlg, IDC_SSRAM, IDC_EXERAM, IDC_SSRAM);
			Refresh(false);
			UpdateStaticText();
			break;
		case IDC_EXERAM:
			ConvOption = 1;
			CheckRadioButton(hDlg, IDC_SSRAM, IDC_EXERAM, IDC_EXERAM);
			Refresh(false);
			UpdateStaticText();
			break;
		case IDC_ABOUT:
			DialogBox((HINSTANCE)GetModuleHandle(NULL), (LPCTSTR)IDD_ABOUTBOX, hDlg, (DLGPROC)About);
		default:
			return false;
		}
		break;
	default:
		return false;
	}

	return false;
}

void Refresh(bool firstChanged)
{
	if(!canupdate)
		return;

	canupdate = false;

	GetWindowText(hPsExeLoad, buf, 255);
	PsExeLoadAddress = strtoul(buf, NULL, 16);
	GetWindowText(hSsLoad, buf, 255);
	EpsxeRamAddress = strtoul(buf, NULL, 16);

	if(firstChanged) // Get the value for the second
	{
		GetWindowText(hConv1, buf, 255);
		Conv1Value = strtoul(buf, NULL, 16);
		Conv2Value = Convert(Conv1Value);
		wsprintf(buf, "%08X", Conv2Value);
		SetWindowText(hConv2, buf);
	}
	else // Update the first
	{
		GetWindowText(hConv2, buf, 255);
		Conv2Value = strtoul(buf, NULL, 16);
		Conv1Value = ConvertBack(Conv2Value);
		wsprintf(buf, "%08X", Conv1Value);
		SetWindowText(hConv1, buf);
	}

	canupdate = true;
}

unsigned int Convert(unsigned int firstValue)
{
	if(ConvOption == 0) // SS->RAM
		return SavestateToRam(firstValue, EpsxeRamAddress);
	else if(ConvOption == 1) // PS-EXE->RAM
		return PsExeToRam(firstValue, PsExeLoadAddress);
	else
		return 0;
}

unsigned int ConvertBack(unsigned int secondValue)
{
	if(ConvOption == 0) // SS<-RAM
		return RamToSavestate(secondValue, EpsxeRamAddress);
	else if(ConvOption == 1) // PS-EXE<-RAM
		return RamToPsExe(secondValue, PsExeLoadAddress);
	else
		return 0;
}

void UpdateStaticText()
{
	if(ConvOption >= ConvStrCount)
		return;
	SetWindowText(hConv1Text, ConvStrings[ConvOption*2]);
	SetWindowText(hConv2Text, ConvStrings[ConvOption*2+1]);
}

unsigned int PsExeToRam(unsigned int PsExeAddr, unsigned int PsExeLoadAddr)
{
	return PsExeAddr + PsExeLoadAddr;
}

unsigned int RamToPsExe(unsigned int RamAddr, unsigned int PsExeLoadAddr)
{
	return RamAddr - PsExeLoadAddr;
}

unsigned int SavestateToRam(unsigned int SsAddr, unsigned int SsRamAddr)
{
	return SsAddr - SsRamAddr + 0x80000000;
}

unsigned int RamToSavestate(unsigned int RamAddr, unsigned int SsRamAddr)
{
	return RamAddr + SsRamAddr - 0x80000000;
}