#include "stdafx.h"
#include <zlib.h>
#include <cstdio>
#include "mstdio.h"
#include "Savestate.h"

using namespace std;

unsigned int OpenEpsxeSavestate(const char* Filename, EPSXE_SAVESTATE* Savestate)
{
	unsigned int version;
	FILE* CompFile = fopen(Filename, "rb");
	if(CompFile == NULL)
		return SS_NOFILE;

	unsigned int gzSize;
	fseek(CompFile, -4, SEEK_END);
	fread(&gzSize, 4, 1, CompFile);
	fclose(CompFile);
	gzFile gzCompFile = gzopen(Filename, "rb");
	unsigned char* gzMem = new unsigned char[gzSize];
	if(gzMem == NULL)
		return SS_MEMERR;
	gzread(gzCompFile, gzMem, gzSize);
	gzclose(gzCompFile);

	MFILE* mFile = mopen(gzSize, "wr");
	if(mFile == NULL)
		return SS_MEMERR;
	mwrite(gzMem, 1, gzSize, mFile);
	delete[] gzMem;	gzMem = NULL;
	mseek(mFile, 0, SEEK_SET);
	
	// Read in header
	mread(Savestate->id, 1, 0x40, mFile);
	if(strncmp("ePSXe", Savestate->id, 5) != 0)
	{
		mclose(mFile);
		return SS_BADFILE;
	}
	
	version = Savestate->id[5];
	Savestate->version = version;

	// PSX
	mread(Savestate->PsxPsx.id, 1, 3, mFile);
	if(strcmp("PSX", Savestate->PsxPsx.id) != 0)
	{
		mclose(mFile);
		return SS_BADFILE;
	}
	mread(&Savestate->PsxPsx.size, 4, 1, mFile);
	Savestate->PsxPsx.data = (unsigned int)mtell(mFile);
	mseek(mFile, Savestate->PsxPsx.size, SEEK_CUR);

	// MEM
	mread(Savestate->PsxMem.id, 1, 3, mFile);
	if(strcmp("MEM", Savestate->PsxMem.id) != 0)
	{
		mclose(mFile);
		return SS_BADFILE;
	}
	mread(&Savestate->PsxMem.size, 4, 1, mFile);
	Savestate->PsxMem.data = (unsigned int)mtell(mFile);
	mseek(mFile, Savestate->PsxMem.size, SEEK_CUR);

	// REG
	mread(Savestate->PsxReg.id, 1, 3, mFile);
	if(strcmp("REG", Savestate->PsxReg.id) != 0)
	{
		mclose(mFile);
		return SS_BADFILE;
	}
	mread(&Savestate->PsxReg.size, 4, 1, mFile);
	Savestate->PsxReg.data = (unsigned int)mtell(mFile);
	mseek(mFile, Savestate->PsxReg.size, SEEK_CUR);

	// IRQ
	mread(Savestate->PsxIrq.id, 1, 3, mFile);
	if(strcmp("IRQ", Savestate->PsxIrq.id) != 0)
	{
		mclose(mFile);
		return SS_BADFILE;
	}
	mread(&Savestate->PsxIrq.size, 4, 1, mFile);
	Savestate->PsxIrq.data = (unsigned int)mtell(mFile);
	mseek(mFile, Savestate->PsxIrq.size, SEEK_CUR);

	// GTE
	mread(Savestate->PsxGte.id, 1, 3, mFile);
	if(strcmp("GTE", Savestate->PsxGte.id) != 0)
	{
		mclose(mFile);
		return SS_BADFILE;
	}
	mread(&Savestate->PsxGte.size, 4, 1, mFile);
	Savestate->PsxGte.data = (unsigned int)mtell(mFile);
	mseek(mFile, Savestate->PsxGte.size, SEEK_CUR);

	// CDR
	mread(Savestate->PsxCdr.id, 1, 3, mFile);
	if(strcmp("CDR", Savestate->PsxCdr.id) != 0)
	{
		mclose(mFile);
		return SS_BADFILE;
	}
	mread(&Savestate->PsxCdr.size, 4, 1, mFile);
	Savestate->PsxCdr.data = (unsigned int)mtell(mFile);
	mseek(mFile, Savestate->PsxCdr.size, SEEK_CUR);

	// SIO
	mread(Savestate->PsxSio.id, 1, 3, mFile);
	if(strcmp("SIO", Savestate->PsxSio.id) != 0)
	{
		mclose(mFile);
		return SS_BADFILE;
	}
	mread(&Savestate->PsxSio.size, 4, 1, mFile);
	Savestate->PsxSio.data = (unsigned int)mtell(mFile);
	mseek(mFile, Savestate->PsxSio.size - 0x190, SEEK_CUR); // 0x190 bytes short...don't ask me why

	// MDE
	mread(Savestate->PsxMde.id, 1, 3, mFile);
	if(strcmp("MDE", Savestate->PsxMde.id) != 0)
	{
		mclose(mFile);
		return SS_BADFILE;
	}
	mread(&Savestate->PsxMde.size, 4, 1, mFile);
	Savestate->PsxMde.data = (unsigned int)mtell(mFile);
	mseek(mFile, Savestate->PsxMde.size, SEEK_CUR);

	// GPU
	mread(Savestate->PsxGpu.id, 1, 3, mFile);
	if(strcmp("GPU", Savestate->PsxGpu.id) != 0)
	{
		mclose(mFile);
		return SS_BADFILE;
	}
	mread(&Savestate->PsxGpu.size, 4, 1, mFile);
	Savestate->PsxGpu.data = (unsigned int)mtell(mFile);
	mseek(mFile, Savestate->PsxGpu.size, SEEK_CUR);

	// SPU
	mread(Savestate->PsxSpu.id, 1, 3, mFile);
	if(strcmp("SPU", Savestate->PsxSpu.id) != 0)
	{
		mclose(mFile);
		return SS_BADFILE;
	}
	mread(&Savestate->PsxSpu.size, 4, 1, mFile);
	Savestate->PsxSpu.data = (unsigned int)mtell(mFile);
	mseek(mFile, Savestate->PsxSpu.size, SEEK_CUR);

	mclose(mFile); // Return here, not so sure if the rest is optional...
	return SS_SUCCESS;

	// PBO
	mread(Savestate->PsxPbo.id, 1, 3, mFile);
	if(strcmp("PBO", Savestate->PsxPbo.id) != 0)
	{
		mclose(mFile);
		return SS_BADFILE;
	}
	mread(&Savestate->PsxPbo.size, 4, 1, mFile);
	Savestate->PsxPbo.data = (unsigned int)mtell(mFile);

	mclose(mFile);

	return SS_SUCCESS;
}