#include "stdafx.h"

#include <cstdio>
#include "mstdio.h"

using namespace std;

// mopen - Returns NULL on insufficient memory, MFILE* on success
MFILE* mopen(size_t memlength, const char* mode)
{
	unsigned int fmode = 0;
	if(strchr(mode, 'r'))
		fmode |= MFILEREAD;
	if(strchr(mode, 'w'))
		fmode |= MFILEWRITE;

	if(memlength == 0)
		return 0;

	MFILE* mfile = (MFILE*)malloc(sizeof(MFILE));
	if(mfile == 0)
		return 0;
	memset(mfile, 0, sizeof(MFILE));
	mfile->mode = fmode;

	mfile->mem = (void*)malloc(memlength);
	if(mfile->mem == NULL) // Memory could not be allocated
	{
		free(mfile);
		return 0;
	}
	mfile->off = 0;
	mfile->size = memlength;

	return mfile;
}

// Frees a MFILE resource
int mclose(MFILE* stream)
{
	free(stream->mem);
	free(stream);
	return 0;
}

// Returns 0 on a seek, nonzero on error
// Seeks to end of stream if position is greater than stream size
int mseek(MFILE* stream, size_t offset, int origin)
{
	if(stream == NULL)
		return -1;
	if(stream->mem == NULL)
		return -1;

	size_t pos;
	if(origin == SEEK_SET)
		pos = offset;
	else if(origin == SEEK_CUR)
		pos = offset + stream->off;
	else if(origin == SEEK_END)
		pos = offset + stream->size;
	else
		return 1;

	if(pos > stream->size)
		stream->off = stream->size;
	else
		stream->off = pos;
	return 0;
}

// Returns zero on error, number of items read on success
size_t mread(void* buffer, size_t size, size_t count, MFILE* stream)
{
	if(buffer == NULL || stream == NULL)
		return 0;
	if(stream->mem == NULL)
		return 0;

	void* membuf = (void*)((char*)stream->mem + stream->off);
	if((stream->mode & MFILEREAD) == 0)
		return 0;

	size_t maxcount = (stream->size - stream->off) / size;
	if(maxcount < count)
		count = maxcount;

	if(size == 2)
	{
		unsigned short* src = (unsigned short*)membuf;
		unsigned short* dest  = (unsigned short*)buffer;
		for(size_t i = 0; i < count; i++)
			*dest++ = *src++;
	}
	else if(size == 4)
	{
		unsigned int* src = (unsigned int*)membuf;
		unsigned int* dest  = (unsigned int*)buffer;
		for(size_t i = 0; i < count; i++)
			*dest++ = *src++;
	}
	else
		memcpy(buffer, membuf, count*size);

	stream->off += count*size;
	return count;
}

// Returns zero on error, number of items written on success
size_t mwrite(const void* buffer, size_t size, size_t count, MFILE* stream)
{
	if(buffer == NULL || stream == NULL)
		return 0;
	if(stream->mem == NULL)
		return 0;

	void* membuf = (void*)((char*)stream->mem + stream->off);

	if((stream->mode & MFILEWRITE) == 0)
		return 0;

	size_t maxcount = (stream->size - stream->off) / size;
	if(maxcount < count)
		count = maxcount;

	if(size == 2)
	{
		unsigned short* dest = (unsigned short*)membuf;
		unsigned short* src  = (unsigned short*)buffer;
		for(size_t i = 0; i < count; i++)
			*dest++ = *src++;
	}
	else if(size == 4)
	{
		unsigned int* dest = (unsigned int*)membuf;
		unsigned int* src  = (unsigned int*)buffer;
		for(size_t i = 0; i < count; i++)
			*dest++ = *src++;
	}
	else
		memcpy(membuf, buffer, count*size);

	stream->off += count*size;
	return count;
}

// Returns -1 on error, memory position on success
size_t mtell(MFILE* stream)
{
	if(stream == NULL)
		return -1;
	if(stream->mem == NULL)
		return -1;
	return stream->off;
}

// Returns -1 on error, 0 on not eof, nonzero on eof
int meof(MFILE* stream)
{
	if(stream == NULL)
		return -1;
	if(stream->mem == NULL)
		return -1;
	return (stream->off == stream->size);
}

// Returns the memory size on success, -1 on error
size_t msize(MFILE* stream)
{
	if(stream == NULL)
		return -1;
	if(stream->mem == NULL)
		return -1;
	return stream->size;
}